/*
 * PSEmu linux Null Sound routines.
 *
 * By: linuzappz <linuzappz@hotmail.com>
 *
 */

#include <stdio.h>
#include <stdlib.h>

typedef char HWND;

#include "PSEmu Plugin Defs.h"
#include "Decode XA.h"

static char *LibName = "LinuzAppz SpuNull Driver";

const unsigned char version = 1; // PSEmu 1.x library
const unsigned char revision = 1;
const unsigned char build = 0;

static int spu_sbaddr;
static short spureg[(0x1e00-0x1c00)/2];
static short *spumem;

char *PSEgetLibName(void) {
    return LibName;
}

unsigned long PSEgetLibType(void) {
    return PSE_LT_SPU;
}

unsigned long PSEgetLibVersion(void) {
    return version<<16|revision<<8|build;
}

long SPUinit(void) {
    spumem = (short *)malloc(512*1024);
    if (spumem == NULL) return -1;

    return 0;
}

long SPUshutdown(void) {
    if (spumem != NULL) {
	free(spumem);
	spumem = NULL;
    }

    return 0;
}

long SPUopen(void) {
    return 0;
}

long SPUclose(void) {
    return 0;
}

// New Interface

void SPUwriteRegister(unsigned long reg, unsigned short val) {
    spureg[(reg-0x1f801c00)/2] = val;
    switch(reg) {
	case 0x1f801da6: // spu sbaddr
    	    spu_sbaddr = val * 8;
    	    break;
	case 0x1f801da8: // spu data
	    spumem[spu_sbaddr/2] = (short)val;
	    spu_sbaddr+=2;
	    if (spu_sbaddr > 0x7ffff) spu_sbaddr = 0;
    	    break;
    }
}

unsigned short SPUreadRegister(unsigned long reg) {
    switch (reg){
	case 0x1f801da6: // spu sbaddr
    	    return spu_sbaddr / 8;
	case 0x1f801da8: // spu data
	    {
	    int ret = spumem[spu_sbaddr/2];
	    spu_sbaddr+=2;
	    if (spu_sbaddr > 0x7ffff) spu_sbaddr = 0;
	    return ret;
	    }
	default:
	    return spureg[(reg-0x1f801c00)/2];
    }
    return 0;
}

unsigned short SPUreadDMA(void) {
    int ret = spumem[spu_sbaddr/2];
    spu_sbaddr+=2;
    if (spu_sbaddr > 0x7ffff) spu_sbaddr = 0;
    return ret;
}

void SPUwriteDMA(unsigned short val) {
    spumem[spu_sbaddr/2] = (short)val;
    spu_sbaddr+=2;
    if (spu_sbaddr > 0x7ffff) spu_sbaddr = 0;
}

void SPUplayADPCMchannel(xa_decode_t *xap) {
}

// Old Interface

unsigned short SPUgetOne(unsigned long val) {
    if (val > 0x7ffff) return 0;
    return spumem[val/2];
}

void SPUputOne(unsigned long val, unsigned short data) {
    if (val > 0x7ffff) return;
    spumem[val/2] = data;
}

void SPUsetAddr(unsigned char ch, unsigned short waddr) {
}

void SPUsetPitch(unsigned char ch, unsigned short pitch) {
}

void SPUsetVolumeL(unsigned char ch, short vol) {
}

void SPUsetVolumeR(unsigned char ch, short vol) {
}

void SPUstartChannels1(unsigned short channels) {
}

void SPUstartChannels2(unsigned short channels) {
}

void SPUstopChannels1(unsigned short channels) {
}

void SPUstopChannels2(unsigned short channels) {
}

long SPUtest(void) {
    return 0;
}

long SPUconfigure(void) {
    return 0;
}

void SPUabout(void) {
}

